#
# Author: Michael Fürst
# Version: 1.0
#

require 'BotnetV2/Network'
require 'securerandom'

module BotnetV2
  class Client
    def initialize(password, client_id, host, port = 8008, ssl = true)
      @exit = false
      @callbacks = Hash.new
      @onReady = Hash.new
      @onReady['client_onReady'] = client_id
      @onReady['verify'] = password

      network = BotNetwork.new
      network.connect host, port if ssl
      network.connect_no_ssl host, port unless ssl

      @connection = BotConnection.new network.socket, network
      @t = Thread.start do
        @connection.on_message_handler_loop(Proc.new do |msg| on_message(msg) end)
      end

      @connection.send @onReady
    end

    def net_status(callback)
      message = Hash.new
      message['status'] = SecureRandom.hex
      @callbacks[message['status']] = callback
      @connection.send message
    end

    def execute (task_src, callback)
      message = Hash.new
      message['onWork'] = Hash.new
      message['onWork']['task_id'] = SecureRandom.hex
      message['onWork']['clientId'] = @onReady['client_onReady']
      message['onWork']['src'] = task_src
      @callbacks[message['onWork']['task_id']] = callback
      @connection.send message
    end

    def on_message (message)
      if message['onResult'] != nil
        error = false
        res = message['onResult']['result']
        if message['onResult']['error'] != nil
          res = message['onResult']['error']
          error = true if message['onResult']['error']
        end
        @callbacks[message['onResult']['task_id']].call(res, error)
      end
      if message['status'] != nil
        @callbacks[message['status']['task_id']].call(message['status'])
      end
    end

    def exit!
      begin
        @connection.close
      rescue => e
        # nothing
      ensure
        @t.kill
      end
      message = Hash.new
      message['client_onDisconnect'] = @onReady['client_onReady']
      @connection.send message
    end
  end
end
